# !/usr/bin/python
# -*- coding: cp1252 -*-
#
##################################################################################
#
#    Copyright 2016 Félix Brezo and Yaiza Rubio (i3visio, contacto@i3visio.com)
#
#    This file is part of Torfy. You can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##################################################################################

import ConfigParser

import os
import sys

def changePermissionsRecursively(path, uid, gid):
    """
        Function to recursively change the user id and group id. It sets 700 
        permissions.
    """
    os.chown(path, uid, gid)
    for item in os.listdir(path):
        itempath = os.path.join(path, item)
        if os.path.isfile(itempath):
            # Setting owner
            try:
                os.chown(itempath, uid, gid)
            except Exception, e:
                # If this crashes it may be because we are running the 
                # application in Windows systems, where os.chown does NOT work.
                pass
            # Setting permissions
            os.chmod(itempath, 0600) 
        elif os.path.isdir(itempath):
            # Setting owner
            try:
                os.chown(itempath, uid, gid)
            except Exception, e:
                # If this crashes it may be because we are running the 
                # application in Windows systems, where os.chown does NOT work.
                pass
            # Setting permissions
            os.chmod(itempath, 6600)             
            # Recursive function to iterate the files
            changePermissionsRecursively(itempath, uid, gid)

def getConfigPath(configFileName = None):
    """
        Auxiliar function to get the configuration path depending on the system.
    """
    if configFileName != None:
        # Returning the path of the configuration file
        if sys.platform == 'win32':
            return os.path.expanduser(os.path.join('~\\', 'Torfy', configFileName))
        else:
            return os.path.expanduser(os.path.join('~/', '.config', 'Torfy', configFileName))
    else:
        # Returning the path of the configuration folder
        if sys.platform == 'win32':
            return os.path.expanduser(os.path.join('~\\', 'Torfy'))
        else:
            return os.path.expanduser(os.path.join('~/', '.config', 'Torfy'))

def getConfiguration(configPath = None):
    """
        Reading the configuration file to look for where the Tor Browser is running.
        
        :return: A json containing the information stored in the .cfg file.
    """
    if configPath == None:
        # If a current.cfg has not been found, creating it by copying from default
        configPath = getConfigPath("browser.cfg")

    # Checking if the configuration file exists
    if not os.path.exists(configPath):
        try:
            # Copy the data from the default folder
            defaultConfigPath = getConfigPath(os.path.join("default", "browser.cfg"))
     
            with open(configPath, "w") as oF:
                with open(defaultConfigPath) as iF:
                    cont = iF.read()
                    oF.write(cont)    
                                                
        except Exception, e:
            print "WARNING. No configuration file could be found and the default file was not found either, so configuration will be set as default."
            print str(e)
            print
            # Storing configuration as default
            info = {}
            info["host"] = "127.0.0.1"
            info["port"] = "9150"
            return info
    try:
        # Reading the configuration file
        config = ConfigParser.ConfigParser()
        config.read(configPath)

        info = {}

        # Iterating through all the sections, which contain the platforms
        for conf in config.sections():
            # Iterating through parametgers
            for (param, value) in config.items(conf):
                info[param] = value
                
    except Exception, e:
        print "WARNING. Something happened when processing the Configuration file (some kind of malform?), so configuration will be set as default."
        print str(e)
        print
        # Storing configuration as default
        info = {}
        info["host"] = "127.0.0.1"
        info["port"] = "9150"
        return info            

    return info




#!/usr/bin/env python
# -*- coding: cp1252 -*-
#
##################################################################################
#
#    Copyright 2016 Félix Brezo and Yaiza Rubio (i3visio, contacto@i3visio.com)
#
#    This file is part of Torfy. You can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##################################################################################

import configuration
import datetime as dt
import json
import os
import re
import socket
import socks
import sys


def getDomainFromUrl(url):
    """
        Extracting the .onion domain from a URL.
        
        :return: domain as a string.
    """
    domain = re.findall("https?://([a-zA-Z0-9]+\.onion)/.*", url)[0]
    return domain

def rebuildHTMLContent(linesArray):
    """ 
        Grabbing a list of lines and rebuilding the HTML as a string.
        
        :return aux: The HTML string.
    """
    aux = ""
    for l in linesArray:
        aux+= l +"\n"
    return aux

def createDataStructure(content):
    """
        This method receives a response, including headers, and creates the appropriate structure.
        :param url: The URL to be recovered.
        :param content: The content of the response.                
        
        :return: A json.        
    """
    aux = {}
    aux["headers"] = {}
    aux["content"] = ""
    for i, line in enumerate(content.splitlines()):
        if i == 0:
            aux["procotol"] = line.split()[0]
            aux["code"] = line.split()[1]
        elif line != "":
            header =  line.split(": ")[0]
            aux["headers"][header] = line[line.find(": ")+2:]
        else:
            aux["content"] += rebuildHTMLContent(content.splitlines()[i+1:])
            break  
    return aux

        
def grabOnionUrl(url):
    """ 
        Function that abstracts capturing an onion URL.
        :param url: The Tor URL to be processed.
        :return:    The HTML content.
    """
    # Configuring default proxy
    try:
        # Grabbing the configuration options from default.        
        info = configuration.getConfiguration()
        
        # Configuring the socket
        socks.setdefaultproxy(socks.PROXY_TYPE_SOCKS5, info["host"], int(info["port"]), True)
        s = socks.socksocket()
        
        # Extracting the domain from the URL
        domain = getDomainFromUrl(url)        
        s.connect((domain, 80))

        message = 'GET ' + url + ' HTTP/1.0\r\n\r\n'
        s.sendall(message)
        
        data = ""
        while True:
            reply = s.recv(4096)

            if not reply:
                break    
            else:
                data += reply    
        
        # Processing data
        response = createDataStructure(data)

        response["time"] = str(dt.datetime.now())
        response["domain"] = domain

        # TO-DO: Perform additional processing here

        return response
        
    except socks.ProxyConnectionError, sPCE:
        print "ERROR socks.ProxyConnectionError: Something seems to be wrong with the Tor Bundler."
        print str(sPCE)    
        sys.exit()

def test():
    print "test"



# -*- coding: cp1252 -*-
#
##################################################################################
#
#    Copyright 2016 Félix Brezo and Yaiza Rubio (i3visio, contacto@i3visio.com)
#
#    This file is part of OSRFramework. You can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##################################################################################

import torfy

WELCOME_TEXT = """
 _____           __       
|_   _|__  _ __ / _|_   _           What's this?  
  | |/ _ \| '__| |_| | | |          ------------
  | | (_) | |  |  _| |_| |          
  |_|\___/|_|  |_|  \__, |          This script is part of Torfy, a GPLv3+
                    |___/           library to deal with Tor connections.

                Version:      Torfy """ + torfy.__version__ + """
                Created by:   Felix Brezo and Yaiza Rubio, (i3visio)

"""




# !/usr/bin/python
# -*- coding: cp1252 -*-
#
##################################################################################
#
#    Copyright 2016 Félix Brezo and Yaiza Rubio (i3visio, contacto@i3visio.com)
#
#    This file is part of Torfy. You can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##################################################################################

import os
import sys

HERE = os.path.abspath(os.path.dirname(__file__))

# Importing the local scrips for the setup and taking the new version number
import torfy
NEW_VERSION = torfy.__version__

import torfy.configuration as configuration

# Depending on the place in which the project is going to be upgraded
from setuptools import setup
try:
    raise Exception('Trying to load the markdown manually!')    
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read_md = lambda f: open(f, 'r').read()
except Exception:
    read_md = lambda f: open(f, 'r').read()

# Reading the .md file    
try:
    long_description = read_md(os.path.join(HERE,"README.md"))
except:
    long_description = ""


# Creating the application path
applicationPath = configuration.getConfigPath()
applicationPathDefaults = os.path.join(applicationPath, "default")

# Copying the default configuration files.
if not os.path.exists(applicationPathDefaults):
    os.makedirs(applicationPathDefaults) 
	
# Launching the setup
setup(    name="torfy",
    version=NEW_VERSION,
    description="Torfy - A set of GPLv3+ libraries to deal with Tor connections.",
    author="Felix Brezo and Yaiza Rubio",
    author_email="contacto@i3visio.com",
    url="http://github.com/i3visio/torfy",
    license="COPYING",
    keywords = "python osint harvesting networking tor privacy",
    scripts= [
        "scripts/onionGet.py",            
        "scripts/onionsFromFile.py",            
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)', 
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python :: 2.7',
        'Intended Audience :: Developers',   
        'Intended Audience :: Information Technology',      
        'Intended Audience :: Science/Research',
        'Intended Audience :: Telecommunications Industry',
        'Natural Language :: English',
        'Topic :: Communications',   
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search',  
        'Topic :: Text Processing :: Markup :: HTML'                                 
    ],    
    packages=[
        "torfy", 
    ],
    long_description=long_description,
    install_requires=[
        "argparse",
        "pysocks",
    ],    
)

############################
### Creating other files ###
############################
try:
    configuration.changePermissionsRecursively(applicationPath, int(os.getenv('SUDO_UID')), int(os.getenv('SUDO_GID')))              
except:
    # Something happened with the permissions... We omit this.
    pass

files_to_copy= {
    applicationPath : [
    ],
    applicationPathDefaults : [
        os.path.join("config", "browser.cfg"),
    ]
}

# Iterating through all destinations to write the info
for destiny in files_to_copy.keys():
    # Grabbing each source file to be moved
    for sourceFile in files_to_copy[destiny]:
        fileToMove = os.path.join(HERE,sourceFile)

        # Choosing the command depending on the SO
        if sys.platform == 'win32':
            cmd = "copy \"" + fileToMove + "\" \"" + destiny + "\""
        elif sys.platform == 'linux2' or sys.platform == 'darwin':   
            cmd = "sudo cp \"" + fileToMove + "\" \"" + destiny + "\""
        #print cmd
        output = os.popen(cmd).read()    

print    



/bin/env python
# -*- coding: cp1252 -*-
#
##################################################################################
#
#    Copyright 2016 Félix Brezo and Yaiza Rubio (i3visio, contacto@i3visio.com)
#
#    This file is part of Torfy. You can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##################################################################################

'''
onionsFromFile.py Copyright (C) F. Brezo and Y. Rubio (i3visio) 2016
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under certain conditions.
For details, run:
    python onionsFromFile.py --license
'''
__author__ = "Felix Brezo, Yaiza Rubio "
__copyright__ = "Copyright 2016, i3visio"
__credits__ = ["Felix Brezo", "Yaiza Rubio"]
__license__ = "GPLv3+"
__version__ = "v0.1.0"
__maintainer__ = "Felix Brezo, Yaiza Rubio"
__email__ = "contacto@i3visio.com"

import argparse
import os
import json
import torfy.banner as banner
import torfy.torwrapper as torwrapper
import datetime as dt


def main(args):
    """
        Main function
    """
    urls = []
    
    # Grabbing all possible URL
    with open(args.file) as iF:
        urls = iF.read().splitlines()

    # Creating the output folder if it does not exist.
    if not os.path.exists(args.output_folder):
        os.makedirs(args.output_folder)

    for url in urls:        
        print dt.datetime.now(), ":\t", url
        # Leemos contenido
        response = tor.grabOnionUrl(url)

        # Storing the full processed response
        with open(os.path.join(args.output_folder, response["domain"] +".json"), "w") as oF:
            oF.write(json.dumps(response, indent = 2))
        # Storing only the HTML
        with open(os.path.join(args.output_folder, response["domain"] +".html"), "w") as oF:
            oF.write(response["content"])


def getParser():
    parser = argparse.ArgumentParser(description='onionsFromFile.py - Grabbing Tor URLs from a file.', prog='onionsFromFile.py', epilog='Check the README.md file for further details on the usage of this program or follow us on Twitter in <http://twitter.com/i3visio>.', add_help=False)
    parser._optionals.title = "Input options (one required)"

    # Defining the mutually exclusive group for the main options
    groupMainOptions = parser.add_mutually_exclusive_group(required=True)
    # Adding the main options
    groupMainOptions.add_argument('--license', required=False, action='store_true', default=False, help='shows the GPLv3+ license and exists.')
    groupMainOptions.add_argument('-f', '--file', metavar='<path>', action='store', help = 'path to the file (one URL per line).')

    # Configuring the processing options
    groupProcessing = parser.add_argument_group('Processing arguments', 'Configuring the way in which torfy will process the URL.')
    groupProcessing.add_argument('-o', '--output_folder', metavar='<path_to_output_folder>', required=False, default = './results', action='store', help='output folder for the generated documents. While if the paths does not exist, usufy.py will try to create; if this argument is not provided, usufy will NOT write any down any data. Check permissions if something goes wrong.')

    # About options
    groupAbout = parser.add_argument_group('About arguments', 'Showing additional information about this program.')
    groupAbout.add_argument('-h', '--help', action='help', help='shows this help and exists.')
    groupAbout.add_argument('--version', action='version', version='%(prog)s ' +__version__, help='shows the version of the program and exists.')

    return parser

if __name__ == "__main__":
    print banner.WELCOME_TEXT
    
    # Grabbing the parser
    parser = getParser()

    args = parser.parse_args()

    # Calling the main function
    main(args)   
